@@ -7,11 +7,8 @@
 #include <string_view>
 
 // If we don't include it here we will get undefined reference link error for fmt::formatter<fmt::string_view>
-#if FMT_VERSION >= 80000
-#    include <fmt/format.h>
-#endif
+#include <fmt/format.h>
 
-#include <QtGlobal>
 #include <QByteArray>
 #include <QLatin1String>
 #include <QStringView>
@@ -41,11 +38,11 @@ namespace tremotesf::impl {
         return fmt::format_to(ctx.out(), "{}::{}::{}", meta.scope(), meta.enumName(), key);
     }
 
-    fmt::format_context::iterator formatQEnum(const QMetaEnum& meta, qint64 value, fmt::format_context& ctx) {
+    fmt::format_context::iterator formatQEnum(const QMetaEnum& meta, std::intmax_t value, fmt::format_context& ctx) {
         return formatQEnumImpl(meta, value, ctx);
     }
 
-    fmt::format_context::iterator formatQEnum(const QMetaEnum& meta, quint64 value, fmt::format_context& ctx) {
+    fmt::format_context::iterator formatQEnum(const QMetaEnum& meta, std::uintmax_t value, fmt::format_context& ctx) {
         return formatQEnumImpl(meta, value, ctx);
     }
 }
@@ -68,38 +65,35 @@ namespace {
 }
 
 namespace fmt {
-    format_context::iterator formatter<QString>::format(const QString& string, format_context& ctx) FORMAT_CONST {
+    format_context::iterator formatter<QString>::format(const QString& string, format_context& ctx) const {
         return formatter<string_view>::format(toFmtStringView(string.toUtf8()), ctx);
     }
 
-    format_context::iterator formatter<QStringView>::format(const QStringView& string, format_context& ctx)
-        FORMAT_CONST {
+    format_context::iterator formatter<QStringView>::format(const QStringView& string, format_context& ctx) const {
         return formatter<string_view>::format(toFmtStringView(string.toUtf8()), ctx);
     }
 
-    format_context::iterator formatter<QLatin1String>::format(const QLatin1String& string, format_context& ctx)
-        FORMAT_CONST {
+    format_context::iterator formatter<QLatin1String>::format(const QLatin1String& string, format_context& ctx) const {
         return formatter<string_view>::format(std::string_view(string.data(), static_cast<size_t>(string.size())), ctx);
     }
 
-    format_context::iterator formatter<QByteArray>::format(const QByteArray& array, format_context& ctx) FORMAT_CONST {
+    format_context::iterator formatter<QByteArray>::format(const QByteArray& array, format_context& ctx) const {
         return formatter<string_view>::format(toFmtStringView(array), ctx);
     }
 
 #if QT_VERSION_MAJOR >= 6
-    format_context::iterator formatter<QUtf8StringView>::format(const QUtf8StringView& string, format_context& ctx)
-        FORMAT_CONST {
+    format_context::iterator
+    formatter<QUtf8StringView>::format(const QUtf8StringView& string, format_context& ctx) const {
         return formatter<string_view>::format(string_view(string.data(), static_cast<size_t>(string.size())), ctx);
     }
 
-    format_context::iterator formatter<QAnyStringView>::format(const QAnyStringView& string, format_context& ctx)
-        FORMAT_CONST {
+    format_context::iterator
+    formatter<QAnyStringView>::format(const QAnyStringView& string, format_context& ctx) const {
         return formatter<QString>::format(string.toString(), ctx);
     }
 #endif
 
-    format_context::iterator formatter<std::exception>::format(const std::exception& e, format_context& ctx)
-        FORMAT_CONST {
+    format_context::iterator formatter<std::exception>::format(const std::exception& e, format_context& ctx) const {
         const auto type = tremotesf::typeName(e);
         const auto what = e.what();
         if (auto s = dynamic_cast<const std::system_error*>(&e); s) {
@@ -108,22 +102,21 @@ namespace fmt {
         return fmt::format_to(ctx.out(), "{}: {}", type, what);
     }
 
-    format_context::iterator formatter<std::system_error>::format(const std::system_error& e, format_context& ctx)
-        FORMAT_CONST {
+    format_context::iterator
+    formatter<std::system_error>::format(const std::system_error& e, format_context& ctx) const {
         return formatSystemError(tremotesf::typeName(e), e, ctx);
     }
 
 #ifdef Q_OS_WIN
-    format_context::iterator formatter<winrt::hstring>::format(const winrt::hstring& str, format_context& ctx)
-        FORMAT_CONST {
+    format_context::iterator formatter<winrt::hstring>::format(const winrt::hstring& str, format_context& ctx) const {
         return formatter<QString>::format(
             QString::fromWCharArray(str.data(), static_cast<QString::size_type>(str.size())),
             ctx
         );
     }
 
-    format_context::iterator formatter<winrt::hresult_error>::format(const winrt::hresult_error& e, format_context& ctx)
-        FORMAT_CONST {
+    format_context::iterator
+    formatter<winrt::hresult_error>::format(const winrt::hresult_error& e, format_context& ctx) const {
         const auto code = e.code().value;
         return fmt::format_to(
             ctx.out(),
