@@ -54,6 +54,13 @@ namespace tremotesf {
         SETTINGS_PROPERTY_TRIVIAL(bool, showMainWindowWhenAddingTorrent, setShowMainWindowWhenAddingTorrent)
         SETTINGS_PROPERTY_TRIVIAL(bool, showAddTorrentDialog, setShowAddTorrentDialog)
 
+        SETTINGS_PROPERTY_TRIVIAL(
+            bool, mergeTrackersWhenAddingExistingTorrent, setMergeTrackersWhenAddingExistingTorrent
+        )
+        SETTINGS_PROPERTY_TRIVIAL(
+            bool, askForMergingTrackersWhenAddingExistingTorrent, setAskForMergingTrackersWhenAddingExistingTorrent
+        )
+
         SETTINGS_PROPERTY_TRIVIAL(bool, isTorrentsStatusFilterEnabled, setTorrentsStatusFilterEnabled)
         SETTINGS_PROPERTY_TRIVIAL(TorrentsProxyModel::StatusFilter, torrentsStatusFilter, setTorrentsStatusFilter)
 
