@@ -9,9 +9,7 @@
 
 #include "addtorrenthelpers.h"
 
-#include "fileutils.h"
 #include "settings.h"
-#include "log/log.h"
 #include "rpc/rpc.h"
 #include "rpc/servers.h"
 #include "rpc/serversettings.h"
@@ -44,23 +42,6 @@ namespace tremotesf {
         };
     }
 
-    void deleteTorrentFile(const QString& filePath, bool moveToTrash) {
-        try {
-            if (moveToTrash) {
-                try {
-                    moveFileToTrash(filePath);
-                } catch (const QFileError& e) {
-                    warning().logWithException(e, "Failed to move torrent file to trash");
-                    deleteFile(filePath);
-                }
-            } else {
-                deleteFile(filePath);
-            }
-        } catch (const QFileError& e) {
-            warning().logWithException(e, "Failed to delete torrent file");
-        }
-    }
-
     QDialog* askForMergingTrackers(Torrent* torrent, std::vector<std::set<QString>> trackers, QWidget* parent) {
         auto* const settings = Settings::instance();
         QMessageBox* messageBox{};
