@@ -5,17 +5,19 @@
 #include "downloaddirectoriesmodel.h"
 
 #include <algorithm>
-#include <map>
 
 #include <QApplication>
 #include <QStyle>
 
 #include "rpc/pathutils.h"
-#include "rpc/torrent.h"
-#include "ui/itemmodels/modelutils.h"
 #include "rpc/rpc.h"
-#include "torrentsproxymodel.h"
+#include "rpc/torrent.h"
+#include "rpc/serversettings.h"
+#include "ui/itemmodels/modelutils.h"
 #include "ui/screens/mainwindow/downloaddirectoriesmodel.h"
+#include "torrentsproxymodel.h"
+
+#include "desktoputils.h"
 
 namespace tremotesf {
     QVariant DownloadDirectoriesModel::data(const QModelIndex& index, int role) const {
@@ -26,8 +28,9 @@ namespace tremotesf {
         switch (role) {
         case DirectoryRole:
             return item.directory;
-        case Qt::DecorationRole:
-            return QApplication::style()->standardIcon(QStyle::SP_DirIcon);
+        case Qt::DecorationRole: {
+            return desktoputils::standardDirIcon();
+        }
         case Qt::DisplayRole:
         case Qt::ToolTipRole:
             if (item.directory.isEmpty()) {
@@ -86,9 +89,7 @@ namespace tremotesf {
             std::vector<DownloadDirectoriesModel::DirectoryItem>& newItems,
             const DownloadDirectoriesModel::DirectoryItem& item
         ) override {
-            return std::find_if(newItems.begin(), newItems.end(), [&](const auto& newItem) {
-                return newItem.directory == item.directory;
-            });
+            return std::ranges::find(newItems, item.directory, &DownloadDirectoriesModel::DirectoryItem::directory);
         }
 
         bool updateItem(
@@ -114,9 +115,7 @@ namespace tremotesf {
         directories.push_back({.directory = {}, .displayDirectory = {}, .torrents = rpc()->torrentsCount()});
         for (const auto& torrent : rpc()->torrents()) {
             const QString& directory = torrent->data().downloadDirectory;
-            auto found = std::find_if(directories.begin(), directories.end(), [&](const auto& item) {
-                return item.directory == directory;
-            });
+            auto found = std::ranges::find(directories, directory, &DirectoryItem::directory);
             if (found == directories.end()) {
                 directories.push_back(
                     {.directory = directory,
