@@ -7,6 +7,10 @@
 
 #include <QDialog>
 
+#include "coroutines/scope.h"
+
+class QLabel;
+
 namespace tremotesf {
     class Rpc;
 
@@ -15,6 +19,11 @@ namespace tremotesf {
 
     public:
         explicit ServerStatsDialog(Rpc* rpc, QWidget* parent = nullptr);
+
+    private:
+        Coroutine<> getDownloadDirFreeSpace(Rpc* rpc, QLabel* freeSpaceField);
+
+        CoroutineScope mFreeSpaceCoroutineScope{};
     };
 }
 
